% Encode a set of bit in vectors format (either -1/1 or 0/1 values)
% into a compact uint8 or uint64 representation
% 
% Usage: C = compact_bits(X, fmt)
% Usage: C = compact_bits(X, 'uint64')
% where
%   X        input bit vector matrix
%   fmt      output format: uint8/uint64
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. 
% See http://www.cecill.info/licences.en.html
%
% This package was written by Raghavendran Balu, Teddy Furon, Herve Jegou
% Copyright (C) INRIA
% Last change: February 2014.

function C = compact_bits(X, fmt)

d = size (X, 1);
n = size (X, 2);

if nargin < 2, fmt = 'uint8'; end
assert (max(X(:)) <= 1);

if strcmp (fmt, 'uint8')
   dcode = 8;
else dcode = 64; 
end

% Construct a matrix to help the binarization
m = 2.^(0:dcode-1);
M = m;
for i = 2:ceil(d/dcode)
  M = blkdiag (M, m);
end
M = M(:,1:d);

  
if strcmp (fmt, 'uint8')
  if min(X(:)) == -1
    C = uint8 (M * (X+1)/2);
  else
    C = uint8 (M * X);
  end 
  
elseif strcmp (fmt, 'uint64')
  if min(X(:)) == -1
    C = uint64 (M * (X+1)/2);
  else
    C = uint64 (M * X);
  end
end

end

