% Configuration object use by experiment function
% 
% Usage: cfg = config()
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. 
% See http://www.cecill.info/licences.en.html
%
% This package was written by Raghavendran Balu, Teddy Furon, Herve Jegou
% Copyright (C) INRIA
% Last change: February 2014. 

function cfg = config()

% Add libraries for yael and antisparse (TODO:configure)

addpath('$BASE/yael/matlab/');
addpath('$BASE/antisparse/');

cfg = struct;
cfg.src = 'sift';                  % dataset type synthetic/sift
cfg.k = 1000;	                        % short-list size
cfg.maxflips = 10;                       % qolsh paramter - stopping criterion for lsh optimization
cfg.rlist = [1 2 5 10 20 50 100 200 500 1000 2000 5000 10000];  % topk evaluation results at k 
cfg.rlist = cfg.rlist (find (cfg.rlist <= cfg.k));

cfg.wantplot = false;                    % toggle plot

cfg.ps = false;                          % compute tightframe project-sign LSH ?
cfg.psr = false;                         % compute random frame project-sign LSH ?
cfg.qolsh = true;                       % compute qoLSH
cfg.anti_sparse = false;                % compute anti sparse ?
cfg.compute_opt = false;                % compute optimal quantization ?

cfg.verbose = 1;                        % print verbosity level  

cfg.n = 100000;                          % number of base vector points
cfg.nq = 1000;                          % number of query points
cfg.nl = 10000;                         % number of training points
cfg.d = 128;                              % number of dimensions
cfg.L_MULT = 2;                         % code size factor - in terms of input dimensions

cfg.data_set = struct;
cfg.data_set.normalize = true;          % normalize the dataset ?
cfg.data_set.power_norm = true;         % power normalize the dataset ?
if ~(strcmp(cfg.src,'synthetic')) && ~(strcmp(cfg.src,'sift'))
    cfg.data_set.powerlaw = 0.5;
else
    cfg.data_set.power_norm = false;
end

cfg.data_set.mean_norm = false;         % mean normalize ?
if strcmp(cfg.src,'sift')
    cfg.data_set.mean_norm = true;
end

cfg.data_set.filter_nan = true;         % filter nan rows ?

cfg.data_set.sift.dir_data = '$BASE/data/';  % Sift dierectory (TODO:configure)
cfg.data_set.sift.version = 'sift/sift';

if cfg.anti_sparse
    cfg.params.h = 1;     % Regularization parameter for anti-sparse coding
end
end
