% Cosine error estimation plotter
% Plots the angle estimation error of LSH+Frame vs qoLSH
% 
% Usage: cosine_error_est()
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. 
% See http://www.cecill.info/licences.en.html
%
% This package was written by Raghavendran Balu, Teddy Furon, Herve Jegou
% Copyright (C) INRIA
% Last change: February 2014. 

function cosine_error_est()
cfg = config();
[x,xq, cfg] = get_dataset(cfg);


fprintf('\nn: %d nq: %d d: %d L:%d\n',cfg.n,cfg.nq,cfg.d, cfg.L);

%------------------------------------------------------------------------
% frame generation
F = gen_frame(cfg.d, cfg.L, 'tightframe');  % generate the projecting frame
F = single (F);

cos_gd = xq'*x;
cos_gd = cos_gd(:);
cos_gdg = round(cos_gd*10)/10; % true grouped to one decimal resolution
cos_gdg_uniq = unique(cos_gdg);
cos_gdg_idx = grp2idx(cos_gdg);

%------------------------------------------------------------------------
% Project and sign for both query and database 
tic
a_x_ps = sign (F' * x);
fprintf ('* Project and sign computed in %.3fs\n', toc);
a_xq_ps = sign (F' * xq);
rx_ps = yael_fvecs_normalize (F * a_x_ps);
rxq_ps = yael_fvecs_normalize (F * a_xq_ps);
cos_ps = rxq_ps'*rx_ps; 
cos_ps = cos_ps(:);
er_ps = cos_ps - cos_gd;
er_mean_ps = accumarray(cos_gdg_idx,er_ps,[],@mean);
er_std_ps = accumarray(cos_gdg_idx,er_ps,[],@std);

%%------------------------------------------------------------------------
%% qoLSH

tic
[a_x_qdes, nflips, x_flc] = qolsh(F, x, cfg.maxflips);
fprintf ('* qoLSH) computed in %.3fs\n', toc);
[a_xq_qdes, xqnflips, xq_flc] = qolsh(F, xq, cfg.maxflips);
rx_qdes = yael_fvecs_normalize (F * a_x_qdes);
rxq_qdes = yael_fvecs_normalize (F * a_xq_qdes);
cos_qdes = rxq_qdes'*rx_qdes; 
cos_qdes = cos_qdes(:);
er_qdes = cos_qdes - cos_gd;
er_mean_qdes = accumarray(cos_gdg_idx,er_qdes,[],@mean);
er_std_qdes = accumarray(cos_gdg_idx,er_qdes,[],@std);

%%------------------------------------------------------------------------
%% Plot

figure('Name','Cosine error estimation')
hold on
errorbar(cos_gdg_uniq,er_mean_ps,er_std_ps,'--+r','LineWidth',3)
errorbar(cos_gdg_uniq,er_mean_qdes,er_std_qdes,'--+b','LineWidth',3)
legend('LSH+frame','qoLSH')
xlabel('True cosine similarity')
ylabel('Error on Cosine similarity')
hold off

end