% Function to get dataset used as base/training/query vectors 
% 
% Usage: [x,xq,cfg] =  get_dataset(cfg)
% where
%   x        base vector
%   xq       query vector
%   cfg      configuration object
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. 
% See http://www.cecill.info/licences.en.html
%
% This package was written by Raghavendran Balu, Teddy Furon, Herve Jegou
% Copyright (C) INRIA
% Last change: February 2014. 

function [x,xq,xl,cfg] =  get_dataset(cfg)
if strcmp(cfg.src,'sift')

    fx = sprintf ('%s/%s%s',cfg.data_set.sift.dir_data,cfg.data_set.sift.version,'_base.fvecs');
    fxq = sprintf ('%s/%s%s',cfg.data_set.sift.dir_data,cfg.data_set.sift.version,'_query.fvecs');
    fxl = sprintf ('%s/%s%s',cfg.data_set.sift.dir_data,cfg.data_set.sift.version,'_learn.fvecs');

    x = fvecs_read(fx);
    xq = fvecs_read (fxq);
    xl = fvecs_read(fxl);
    
elseif strcmp(cfg.src,'synthetic')

    x = single (randn(cfg.d, cfg.n));           % Produce some database random vectors  
    xq = single (randn(cfg.d, cfg.nq));         % Queries
    xl = single (randn(cfg.d, cfg.nl));
end

if cfg.data_set.power_norm
    fprintf('Power law normalization: %0.3f\n',cfg.data_set.powerlaw);
    x = sign (x) .* abs(x).^cfg.data_set.powerlaw;  
    xq = sign (xq) .* abs(xq).^cfg.data_set.powerlaw;
    xl = sign (xl) .* abs(xl).^cfg.data_set.powerlaw;
end

if cfg.data_set.mean_norm
    fprintf('Mean normalization\n');
    xlm = mean(xl,2);
    x = bsxfun(@minus,x,xlm);
    xq = bsxfun(@minus,xq,xlm);
    xl = bsxfun(@minus,xl,xlm);
end

if cfg.data_set.normalize
    fprintf('Length normalization to unit\n');
    x = yael_fvecs_normalize (x);
    xq = yael_fvecs_normalize (xq);
    xl = yael_fvecs_normalize (xl);
end

if cfg.data_set.filter_nan
    x = single(filter_nan(x));
    xq = single(filter_nan(xq));
    xl = single(filter_nan(xl));
end

cfg.n = size(x,2);
cfg.nq = size(xq,2);
cfg.nl = size(xl,2);
cfg.d = size(x,1);
cfg.L = floor(cfg.L_MULT*cfg.d);

end
