% Main function that compares qoLSH with LSH, LSH+frame, Antisparse, Optimal
% 
% Usage: test_qolsh()
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. 
% See http://www.cecill.info/licences.en.html
%
% This package was written by Raghavendran Balu, Teddy Furon, Herve Jegou
% Copyright (C) INRIA
% Last change: February 2014. 

function [result] = test_qolsh()

cfg = config();
[x,xq,xl,cfg] = get_dataset(cfg);

fprintf('\nn: %d nq: %d d: %d L:%d\n',cfg.n,cfg.nq,cfg.d, cfg.L);

%------------------------------------------------------------------------
% frame generation
F = gen_frame(cfg.d, cfg.L, 'tightframe');  % generate the projecting frame
F = single (F);

rF = gen_frame(cfg.d, cfg.L, 'randn');  % generate the projecting frame  
rF = single (rF);


%------------------------------------------------------------------------
% Compute ground-truth
tic
% compute/load the ground-truth

[gnd, gndip] = yael_nn (x, xq);

gndip = -gndip;
fprintf ('* Ground-truth computed in %.3fs\n', toc);

%------------------------------------------------------------------------
% Project and sign (random frame) for both query and database 
if cfg.psr
tic
a_x_ps = sign (rF' * x);
fprintf ('* Project and sign (random frame) computed in %.3fs\n', toc);
a_xq_ps = sign (rF' * xq);
result =  eval_coding(a_x_ps, a_xq_ps, x, xq, rF, gnd, cfg, 'LSH');
end;

%------------------------------------------------------------------------
% Project and sign for both query and database 
if cfg.ps
tic
a_x_ps = sign (F' * x);
fprintf ('* Project and sign computed in %.3fs\n', toc);
a_xq_ps = sign (F' * xq);
result(length(result)+1) =  eval_coding(a_x_ps, a_xq_ps, x, xq, F, gnd, cfg, 'LSH+frame' );
end;

%%------------------------------------------------------------------------
%% qoLSH

if cfg.qolsh
tic
[a_x_qdes, nflips, x_flc] = qolsh(F, x, cfg.maxflips);
fprintf ('* qoLSH computed in %.3fs\n', toc);
[a_xq_qdes, xqnflips, xq_flc] = qolsh(F, xq, cfg.maxflips);
result(length(result)+1) =  eval_coding(a_x_qdes, a_xq_qdes, x, xq, F, gnd, cfg, 'qoLSH' );
end

%------------------------------------------------------------------------
%% Anti sparse computation

if cfg.anti_sparse
tic
     [a_x_as, b_x_as] = binarize_vec (x, F, 'antisparse', cfg.params);
fprintf ('* Anti sparse computation in %.3fs\n', toc);
     [a_xq_as, b_xq_as] = binarize_vec (xq, F, 'antisparse', cfg.params);
     result(length(result)+1) =  eval_coding(a_x_as, a_xq_as, x, xq, F, gnd, cfg, 'antisparse' );
end

%------------------------------------------------------------------------
%% Best possible quantization

if cfg.compute_opt

% List the centroids only for optimal quantizer (NOT for L>20!)

A = single (zeros (cfg.L, 2^cfg.L));      % Make all combinations of +-1
for l=1:cfg.L
A (l, :) = 2 * bitget (0:2^cfg.L-1, l) - 1;
end

C = F * A;                        % Possible reconstruction values
Cn = yael_fvecs_normalize(C);     % Corresponding spherical quantizer
Cnorm = sqrt(sum(C.^2));           

% Best code

idxok = find (Cnorm > 0 * sqrt(cfg.L));
Cn = Cn(:, idxok);
A = A(:, idxok);

tic
[ci_x_opt, ~] = yael_nn (Cn, x);
fprintf ('* Best quantization in %.3fs\n', toc);
[ci_xq_opt, ~] = yael_nn (Cn, xq);


% Corresponding binary codes
a_x_opt = A(:, ci_x_opt);
a_xq_opt = A(:, ci_xq_opt);
result(length(result)+1) =  eval_coding(a_x_opt, a_xq_opt, x, xq, F, gnd, cfg, 'optimal' );
end

%------------------------------------------------------------------------
% Plot recall
if cfg.wantplot
    plot_recall(result,cfg)
end
end